#  aRT : API R - TerraLib                                                
#  Copyright (C) 2003-2005  LEG                                          
#                                                                        
#  This program is free software; you can redistribute it and/or modify  
#  it under the terms of the GNU General Public License as published by  
#  the Free Software Foundation; either version 2 of the License, or     
#  (at your option) any later version.                                   
#                                                                        
#  This program is distributed in the hope that it will be useful,       
#  but WITHOUT ANY WARRANTY; without even the implied warranty of        
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
#  GNU General Public License for more details.                          
#                                                                        
#  You should have received a copy of the GNU Lesser General Public      
#  License along with this library.

setClass("aRTtable", contains = "aRTcomponent")

# generateids is used in static tables for generating ids from the
# layer geometries.
# only external tables are allowed to be built from aRTdb's
setMethod("initialize", "aRTtable", function(.Object, 
                                             parent,
                                             tablename,
										     create = FALSE,
											 type = NULL,
											 generateids = FALSE,
											 id="id",
											 length=16)
{
	if (is.object(parent) || class(parent) == "aRTlayer")
	{
		if(is.null(type)) type = "s"
		if(type == "x") stop("Only aRTdb can create external tables")
		.Object@pointer = .Call("cppLTable",
    	                    parent@pointer,
			                list(tablename = tablename,
						         create = create,
							     type = type,
								 generateids = generateids,
								 id = id,
								 length = as(length, "integer")),
							PACKAGE="aRT")

	}
	else if (is.object(parent) || class(parent) == "aRTdb")
	{
		if(is.null(type)) type = "x"
		if(type != "x") stop("aRTdb can only create external tables")
		
		.Object@pointer = .Call("cppDTable",
    	                    parent@pointer,
			                list(tablename = tablename,
						         create = create,
							     type = type,
								 generateids = generateids,
								 id = id,
								 length = as(length, "integer")),
							PACKAGE="aRT")
	}
	else
	{
    	stop("Argument \"parent\" must be an object of aRTlayer or aRTdb\n")
	}

  	reg.finalizer(.Object@pointer, .aRTremove)
	.Object
})

setGeneric("createRelation",
           function(object,
 		            attr,
					rtable,
					rattr = attr)
           standardGeneric("createRelation"))

setMethod("createRelation","aRTtable",
           function(object,
		            attr,
					rtable,
					rattr = attr)
{
	summrtable = summary(rtable)
	summobject = summary(object)

	if(!is.object(rtable) || class(rtable) != "aRTtable")
		stop("Argument \"rTable\" must be an aRTtable")
	if(!any(summobject$columns == attr))
		stop(paste("Table", summary(table)$name, " does not have attribute \"", attr, "\"", sep = ""))
	if(!any(summrtable$columns == rattr))
		stop(paste("Table", summrtable$name, " does not have attribute \"", rattr, "\"", sep = ""))
	
	if(summobject$type == "external")
		stop("Object can't be an external table")
	
	if(summrtable$type != "external")
		stop("Relation table must be an external table")
	
	.Call("cppCreateRelation", object@pointer,
	                           list(attr=attr,
					                rtable=summrtable$name,
							        rattr=rattr),
								PACKAGE="aRT")
})

#setGeneric("addColumns", function(object, data) standardGeneric("addColumns"))
setGeneric("addRows",    function(object, data) standardGeneric("addRows"))
setGeneric("updateColumns",function(object, data) standardGeneric("updateColumns"))

setMethod("addRows", "aRTtable", function(object, data) {
	if( !is.data.frame(data)) stop("data must be a data.frame")

    id=summary(object)$keys

	if(!any(id[1] == colnames(data)))
	{
	    cat("Using rownames as id\n")
		l=list()
		l[[id]]=rownames(data)
		data=cbind(data, l)
	}
	
	.Call("cppAddRows",
	      object@pointer,
		  list(colnames= colnames(data),
		       length=length(rownames(data)),
		       matrix  = as.matrix(format.data.frame(data))),
			   # format.data.frame to convert the elements to string
		  PACKAGE="aRT")
	return(invisible())
})

setGeneric("createColumn",
           function(object,
                    colname,
		            type="s",
					length=10,
					key=FALSE,
					decimals=3,
					autonumber=FALSE)
		   standardGeneric("createColumn"))

setMethod("createColumn",
          "aRTtable",
		  function(object,
                   colname,
				   type="s",
				   length=10,
				   key=FALSE,
				   decimals=3,
				   autonumber=FALSE) {
	.Call("cppCreateColumn",
	      object@pointer,
		  list(colname=colname,
		       type=type,
			   length=as(length, "integer"),
			   decimals=as(decimals, "integer"),
			   autonumber=autonumber,
			   key=key))
	return(invisible())
})

setMethod("updateColumns", "aRTtable",function(object, data){
	summ = summary(object)
	columns = summ$columns
	keys = summ$keys
	
	# check if some attribute does not belong to the table, and insert the attribute
    colnames = colnames(data)
    classes <- substr(sapply(data, class),1,1)
    for(i in 1:length(colnames))
    {
		if(!any(columns == colnames[i]))
		{
			createColumn(object, colnames[i], type = classes[i], length=max(nchar(as.matrix(data[i]))))
		}
    }

	.Call("cppUpdateColumns",
	      object@pointer,
		  list(colnames= colnames(data),
		       length=length(rownames(data)),
		       matrix  = as.matrix(format.data.frame(data))),
			   # format.data.frame to convert the elements to string
		  PACKAGE="aRT")

	return (invisible())
})

# receives as arguments n>1 lists of data. one of them must be $id
#setMethod("addColumns", "aRTtable", function(object, data) {
#	if( !any(names(data)=="id") ) stop("data must have a list called 'id'")
#	if( !is.character(data$id) )  stop("'id' must be in character format")
#
#	if( !is.null(data$points)   ) data$points   <- NULL
#	if( !is.null(data$polygons) ) data$polygons <- NULL
#	if( !is.null(data$lines)    ) data$lines    <- NULL
    # other geometries...
#
#	other<-names(data[-which(is.element(names(data),"id"))])
#
#	for(i in 1:length(other))
#	{
#		size<-0
#		data_ <- data[[other[i]]]
#
#		if(is.character(data_[1]))
#		{
#			type<-"s"
#	        for(j in 1:length(data))
#	        {
#	            if(nchar(data_[j]) > size)
#	                size <- nchar(data_[j])
#	        }
#	    }
#		else if(is.real(data_[1]))
#			type<-"r"
#		else if(is.integer(data_[1]))
#			type<-"i"
#		else stop(paste("Unknown type of ", other[i]))
#	
#		if( length(data$id) != length(data_) )
#			stop("Data with different lengths")
#	
#		config<-data
#		config$type_<-type
#		config$name_<-other[i]
#		config$size_<-size
#
#		createColumn(object, name=other[i], type=type,length=size)
#	}
#	update(table, data)
#    return(invisible())
#})

setGeneric("getData", function(object) standardGeneric("getData"))

setMethod("getData", "aRTtable", function(object) {
	.Call("cppGetTableData", object@pointer, PACKAGE="aRT")
})


# returns a terralib format string to representing dates
#todate=function(year = 0, month = 0, day = 0, hour = 0, minute = 0, second = 0)
#{
#	return(invisible())
#}

