/**************************************************************************\
 *  aRT : API R - TerraLib                                                *
 *  Copyright (C) 2003-2005  LEG                                          *
 *                                                                        *
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation; either version 2 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU Lesser General Public      *
 *  License along with this library.                                      *
\**************************************************************************/

#include "aRTproxy.h"
#include "SEXPutils.h"

using namespace std;

aRTproxy::aRTproxy(TeLayer* layer, bool loadgeom, bool loadatt)
{
	TeInitQuerierStrategies(); // never forget it!

    TeQuerierParams params(loadgeom, loadatt); 
    params.setParams(layer);

	Querier = new TeQuerier(params);
	Querier->loadInstances();
}

SEXP aRTproxy::GetNext()
{
	TeSTInstance sti;
	TeMultiGeometry mg;

    if( !Querier->fetchInstance(sti) ) return R_NilValue;

	mg = sti.geometries();

	if( mg.hasPolygons() ) return GetPolygonsTeMultiGeometry(mg);
	if( mg.hasLines()    ) return GetLinesTeMultiGeometry(mg);
	if( mg.hasPoints()   ) return GetPointsTeMultiGeometry(mg);
	
	return R_NilValue;
}

aRTproxy::~aRTproxy()
{
	PrintSilent("Deleting one aRTproxy object from memory\n");
	delete Querier;
}
	
// this is the order to appear inside R. S is from Summary
enum aRTproxySenum
{
	aRTproxySClass = 0,
	aRTproxySElements,
	aRTproxySSize
};
					   
SEXP aRTproxy::Summary()
{
    SEXP info, names;

	info    = allocVector(VECSXP, aRTproxySSize);  // List containing the connection attributes
    names   = allocVector(STRSXP, aRTproxySSize);  // Names Vector

	STRING_LIST_ITEM(info, names, aRTproxySClass,    "class",    Class());
    DOUBLE_LIST_ITEM(info, names, aRTproxySElements, "elements", Querier->numElemInstances());

	setAttrib(info, R_NamesSymbol, names); // Set the names of the list

    return info;
}

const string aRTproxy::Print()
{
	stringstream s;

	if (!IsValid()) error("Invalid object!");
		 
	 s << artOBJECT("aRTproxy")   << "\n\n"
       << "Number of elements: " << Querier->numElemInstances()  << endl;

	return s.str();
}

bool aRTproxy::IsValid()
{
	if(Valid && !Querier)
	{
		Valid = false;
	}

	return Valid;
}

